//
//  ContentView.swift
//  Deliverable 1
//
//  Created by Sneh Kothari on 17/09/22.
//

import SwiftUI
import RealityKit
import ARKit

enum Screens {
    case NewView, ViewCard
}
struct ContentView : View {
    @ObservedObject var arWorldMap = RoomMapper()
    @State var screen: Screens = .ViewCard
    var body: some View {
        switch screen {
        case .ViewCard:
            ViewCard(arWorldMap: arWorldMap, screen: $screen)
        case .NewView:
            RoomView(screen: $screen, arWorldMap: arWorldMap, arViewContainer: ARWorldViewContainer(arWorldMap: arWorldMap))
        }
    }
}



#if DEBUG
struct ContentView_Previews : PreviewProvider {
    static var previews: some View {
        ContentView()
    }
}
#endif

//struct ARViewContainer: UIViewRepresentable {
//    let placeObjectView = PlaceObjectController(frame: .zero)
//    func makeUIView(context: Context) -> ARView {
//
//        let arView = placeObjectView
//        return arView
//
//    }
//
//    func updateUIView(_ uiView: ARView, context: Context) {}
//
//}
struct ARWorldViewContainer: UIViewRepresentable {
    @ObservedObject var arWorldMap: RoomMapper
    let placeObjectView = PlaceObjectController(frame: .zero)
    func makeUIView(context: Context) -> ARView {
        if arWorldMap.roomName != "New Room" {
            let config = ARWorldTrackingConfiguration()
            config.planeDetection = [.horizontal]
            config.environmentTexturing = .automatic
            let map = try! arWorldMap.loadWorldMap()
            config.initialWorldMap = map
            let arView = placeObjectView
            arView.session.run(config, options: [.resetTracking, .removeExistingAnchors])
            return arView
        } else {
            let arView = placeObjectView
            return arView
        }
    }
    
    func updateUIView(_ uiView: ARView, context: Context) {}
    
}
