//
//  PlaceObjectController.swift
//  Deliverable 1
//
//  Created by Sneh Kothari on 17/09/22.
//

import UIKit
import RealityKit
import ARKit

class PlaceObjectController: ARView, ARSessionDelegate {
    
    var resolution:CGAffineTransform?
    private var planeEntities: [UUID:ModelEntity] = [:]
    
    
    var pannedEntity:ModelEntity?
    var lastPan = CGPoint.zero
    var materialXPan:Float = 0
    var materialYPan:Float = 0
    var twoFingerPhysicsChanged: Bool = false
    
    
    required init(frame : CGRect) {
        super.init(frame: frame)
        let tapGesture = UITapGestureRecognizer(target: self, action: #selector(handleTap(sender:)))
        self.addGestureRecognizer(tapGesture)
        let pan = UIPanGestureRecognizer(target: self, action:  #selector(handlePan(sender:)))
        pan.minimumNumberOfTouches = 2
        self.addGestureRecognizer(pan)
    }
    
    
    @objc func handleTap(sender: UITapGestureRecognizer) {
        let location = sender .location(in: self)
        let results = self.raycast(from: location, allowing: .estimatedPlane, alignment: .any)
        if let firstResult = results.first {
            let anchor = ARAnchor(name: "Anchor for object", transform: firstResult.worldTransform)
            self.session.add(anchor: anchor)
            let anchorEntity = AnchorEntity(anchor: anchor)
            
            guard let entity = try? Entity.load(
              named: "teapot"
            ) else { fatalError("Error") }
            let modelEntity = entity
            print(modelEntity)
            print(modelEntity.isAnchored)
            print(modelEntity.name)
            anchorEntity.addChild(modelEntity)
            modelEntity.generateCollisionShapes(recursive: true)
            
            installGestures(.all, for: modelEntity).forEach { recognizer in
                recognizer.addTarget(self, action: #selector(handleGesture(sender:)))
            }
            
//            modelEntity.position.y = (modelEntity.model?.mesh.bounds.extents.y)! / 2
            self.scene.addAnchor(anchorEntity)
        }
    }
    
    @MainActor required dynamic init?(coder decoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    func generateMesh() -> MeshResource {
        var mesh:MeshResource
        mesh = .generateSphere(radius: 0.025)
        return mesh
    }
    @objc func handleGesture(sender: UIGestureRecognizer){
        if let panGesture = sender as? EntityTranslationGestureRecognizer {
            print(panGesture.location(in: self))
        }
        if let pintchGesture = sender as? EntityScaleGestureRecognizer {
            print(pintchGesture.scale)
        }
    }
    @objc func handlePan(sender: UIPanGestureRecognizer) {
        switch sender.state {
        case .began:
            lastPan = CGPoint.zero
            let location = sender.location(in: self)
            if let entity = self.entity(at: location) as? ModelEntity {
                if entity.physicsBody?.mode == .dynamic {
                    twoFingerPhysicsChanged = true
                    entity.physicsBody?.mode = .kinematic
                }
                pannedEntity = entity
            }
        case .changed:
            let newTranslation = sender.translation(in: self)
            guard let position = pannedEntity?.position else { return }
            pannedEntity?.position = [position.x - materialXPan,position.y + materialYPan, position.z]
            lastPan = newTranslation
            
        case .ended:
            if twoFingerPhysicsChanged {
                twoFingerPhysicsChanged = false
                pannedEntity = nil
            }
        default: break
        }
    }
    }

extension Entity: HasCollision {
    
}
